-- if the module would be useless, don't load it.
do
	local _, playerClass = UnitClass("player")
	if playerClass ~= "WARRIOR" then return end
end

-- get a reference to the Faceroller module
local MODULE = "Arms Warrior"
local Faceroller = _G["Faceroller"]
local gsi = Faceroller.gsi
local sudden_death
local taste_for_blood
local rend

local options
local skills

--
-- init function
--
local Init = function(opts)

	options = opts

	if opts.rage_dump_threshold == nil then
		opts.rage_dump_threshold = 60
	end

	if opts.ms_enabled == nil then
		opts.ms_enabled = true
	end

	skills = {
		[34428] = 0,		-- Victory Rush
		[772] = 10,			-- Rend
		[5308] = 15,		-- Execute
		[7384] = 5,			-- Overpower
		[12294] = 30,		-- Mortal Strike
		[1464] = 15,		-- Slam
		[78] = 0,			-- Heroic Strike
		[15284] = 0,		-- Cleave
		[46924] = 25,		-- Bladestorm
	}

	-- check if player has all spells in his spell book.
	for k, v in pairs(skills) do
		if Faceroller:HasSpell (k) == false then
			return nil
		end
	end

	-- register for debuffs we want to track:
	-- rend, but only our own
	Faceroller:RegisterDebuff(772, true)

	Faceroller:RegisterBuff(29724)
	Faceroller:RegisterBuff(56638)

	-- for convenience (used below).
	rend = gsi[772]
	sudden_death = gsi[29724]
	taste_for_blood = gsi[56638]

	return skills
end



local NextAttack = function(gcd, spells, buffs, debuffs, myDebuffs)
	-- threshold to suggest a skill that is still on cooldown.
	local th = max (0.5, gcd)
	th = th + 0.1

	local up = UnitPower("player")
	local target_health = UnitHealth("target") / UnitHealthMax("target")

	if target_health >= 0.2 then   --- standard rotation

		if up >= tonumber(options.rage_dump_threshold) and (spells[78].enabled ~= 1 and spells[15284].enabled ~= 1) then
			Faceroller:CornerIndicatorSetColor ("TOPRIGHT", 1, 0, 0, 1)
		else
			Faceroller:CornerIndicatorSetColor ("TOPRIGHT", 0, 0, 0, 0)
		end

		-- Victory Rush
		if spells[34428].usable then
			return 34428
		end
	
		-- rend
		local v = myDebuffs[rend]
		if v.active == false then
			return 772
		end
	
		-- overpower
		if spells[7384].cd < th and spells[7384].usable then
			if buffs[taste_for_blood].active and buffs[taste_for_blood].time_left <= 1.5 then
				return 7384
			elseif buffs[taste_for_blood].active == false then
				return 7384
			end
		end
	
		-- none of the above is ready, suggest heroic strike

		--if up >= options.rage_dump_threshold and (spells[78].enabled ~= 1 and spells[15284].enabled ~= 1) then
		--	return 78
		--end
	
		-- mortal strike
		if spells[12294].cd < th and spells[12294].data < up and options.ms_enabled then
			return 12294
		end
	
		-- execute
		if (spells[5308].usable or buffs[sudden_death].active) and spells[5308].cd < th and up >= spells[5308].data then
			return 5308
		end
	
		-- Taste for Blood duration > 1.5
		if spells[7384].cd < th and spells[7384].usable then
			return 7384
		end
	
		-- Bladestorm
		if spells[46924].cd < th and up >= spells[46924].data then
			return 46924
		end
	
		-- slam
		if spells[1464].cd < th and up >= spells[1464].data then
			return 1464
		end
	else  -- 20% and under rotation
		-- overpower
		if spells[7384].cd < th and spells[7384].usable then
			return 7384
		end

		-- Bladestorm
		if spells[46924].cd < th and up >= spells[46924].data then
			return 46924
		end
	
		-- execute
		if (spells[5308].usable or buffs[sudden_death].active) and spells[5308].cd < th and up >= 30 then
			return 5308
		end	
	end

	return 0
end

local Options = function(cmd)
	local Cmd, Args = strsplit(" ", cmd:lower(), 2)

	if Cmd == "rage" then
		options.rage_dump_threshold = Args
		print (MODULE .. ": rage dump threshold: " .. options.rage_dump_threshold)
	elseif Cmd == "ms" then
		if options.ms_enabled == true then
			options.ms_enabled = false
			print (MODULE .. ": disabled mortal strike")
		else
			options.ms_enabled = true
			print (MODULE .. ": enabled mortal strike")
		end
	else
		print (MODULE ,"Settings:")
		print ("","Mortal Strike Enabled: ", options.ms_enabled)
		print ("","Rage Dump Threshold: ",options.rage_dump_threshold)
		print (MODULE,"Options:")
		print ("","ms - toggle mortal strike usage")
		print ("","rage <X> - set rage dump threshold to X")
	end
end


-- register the module. (the last arg, which we pass as
-- nil here, would be the function to handle module options.
-- see FR_test2.lua for an example on how to do that).
Faceroller:RegisterModule("arms", Init, NextAttack, Options)
